#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <glob.h>
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"

using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using namespace PVO::FITS;
using std::string;


class Match
{
public:
  
  Match(){};
  virtual ~Match(){};
  friend bool operator< (const Match& lhs, const Match& rhs){
    return (lhs.angsep < rhs.angsep);
  };
  void define(std::string n, double r, double d, std::string t,
	      float b, std::string f, float a){
    name=n; ra=r; dec=d; date=t; beam=b; obsfreq=f; angsep=a;
  }
  std::string name;
  double ra;
  double dec;
  std::string date;
  float beam;
  std::string obsfreq;
  float angsep;
};


int main(int argc, char * argv[])
{
  /** 
   * PVOConeSearch is a function to do basic cone searching in PSRFITS files.
   *
   * This program is a command-line program that looks at all PSRFITS
   * files in the current directory (i.e. files that have extensions
   * .fits or .rf) and returns all those that cover sky within a given
   * search radius of a given RA,DEC position. 
   *
   * The format of usage is: <br> 
   * <tt>> pvoconesearch RA DEC RADIUS</tt><br> 
   * RA and DEC can be in either decimal degrees or hms/dms
   * format (i.e.  hh:mm:ss.sss for RA and dd:mm:ss.sss for
   * DEC). Entering <br>
   * <tt>> pvoconesearch -h</tt><br>
   * will give a help message, using PVO::pvoconesearch_usage_text.
   *
   * The output is a list of valid files, their central RA & DEC,
   * their beam size, the central observing frequency, the DATE-OBS
   * keyword (time/date of observation), plus the angular offset [deg]
   * from the query position. The beam size of the FITS file is taken
   * into account, so that the central position can be more than
   * RADIUS away from the query position (as long as the query
   * position is still within the primary beam's FWHM).
   * 
   */


  bool verboseflag = false;

  std::vector<std::string> namelist;
  std::string RAstring,DECstring;
  double radius,RAcentre,DECcentre;
  unsigned int i,numColon;
  switch(argc){
  case 4:
    glob_t globbuf1,globbuf2;
    globbuf1.gl_offs = 0;
    globbuf2.gl_offs = 0;
    glob("*.fits", GLOB_DOOFFS, NULL, &globbuf1);
    glob("*.rf", GLOB_DOOFFS, NULL, &globbuf2);
    namelist.resize(globbuf1.gl_pathc+globbuf2.gl_pathc);
    for(i=0;i<globbuf1.gl_pathc+globbuf1.gl_offs;i++) 
      namelist[i] = globbuf1.gl_pathv[i];
    for(i=0;i<globbuf2.gl_pathc+globbuf2.gl_offs;i++) 
      namelist[globbuf1.gl_pathc+i] = globbuf2.gl_pathv[i];

    RAstring = argv[1];
    numColon=0;
    for(i=0;i<RAstring.size();i++) if(RAstring[i]==':') numColon++;
    if(numColon>0) RAcentre = dmsToDec(RAstring)*15.;
    else RAcentre = atof(argv[1]);
    DECstring = argv[2];
    numColon=0;
    for(i=0;i<DECstring.size();i++) if(DECstring[i]==':') numColon++;
    if(numColon>0) DECcentre = dmsToDec(DECstring);
    else DECcentre = atof(argv[2]);

    radius = atof(argv[3]);
    
    std::cout.setf(std::ios::fixed);
    std::cout << std::setprecision(4)
	      << "Searching with: RA = " << RAcentre
	      << " deg, DEC = " << DECcentre
	      << " deg, Radius = " << radius << " deg\n";

    break;
  default:
    if(argc==2 && strcmp(argv[1],"-h")==0)
      std::cout << pvoconesearch_usage_text;
    else
      std::cerr << pvoconesearch_usage_text_short;
    return 0;
    break;
  }

  std::vector<Match> goodlist;

  for(i=0;i<namelist.size();i++){

    PSRFits fitsfile;
    fitsfile.setVerbosity(verboseflag);
    int status = fitsfile.openFile(namelist[i]);
  
    if(status==0){
      fitsfile.gotoPHU();
    
      std::string raS,decS,date,obsfreqS;
      std::string missing = "---";
      if(fitsfile.getKeyword<std::string>("RA",raS)!=0) raS = missing;
      if(fitsfile.getKeyword<std::string>("DEC",decS)!=0) decS = missing;
      if(fitsfile.getKeyword<std::string>("DATE-OBS",date)!=0) date = missing;
      if(fitsfile.getKeyword<std::string>("OBSFREQ",obsfreqS)!=0) obsfreqS = missing;
      float beamfwhm,bmaj,bmin;
      fitsfile.getKeyword<float>("BMAJ",bmaj);
      fitsfile.getKeyword<float>("BMIN",bmin);
      beamfwhm = 0.5*(bmaj+bmin); // average FWHM, in deg

      double ra = dmsToDec(raS)*15;
      double dec = dmsToDec(decS);

      if(raS!=missing && decS!=missing){
	double angsep = angSep(ra, dec, RAcentre, DECcentre);

	if(angsep < radius+beamfwhm/2.) {
	  Match newmatch;
	  newmatch.define(namelist[i],ra,dec,date,beamfwhm,obsfreqS,angsep);
	  goodlist.push_back(newmatch);
	}

      
      }
    }
  }

  std::stable_sort(goodlist.begin(),goodlist.end());

  int maxsize = 8; // size of "FILENAME"
  for(i=0;i<goodlist.size();i++) 
    maxsize = std::max(maxsize,int(goodlist[i].name.size()));

  std::cout << std::setfill('-')  
	    << std::setw(maxsize+1+15+15+31+11+11+13) << "-"
	    << "\n" << std::setfill(' ') 
	    << std::setw(maxsize) << "FILENAME"  << " "
	    << std::setw(14)      << "RA"        << " "
	    << std::setw(14)      << "DEC"       << " "
	    << std::setw(10)      << "BEAM FWHM" << " "
	    << std::setw(10)      << "OBSFREQ"   << " "
	    << std::setw(30)      << "DATE-OBS"  << " "
	    << std::setw(12)      << "OFFSET"    << "\n"
	    << std::setw(maxsize) << ""          << " "
	    << std::setw(14)      << "[deg]"     << " "
	    << std::setw(14)      << "[deg]"     << " "
	    << std::setw(10)      << "[deg]"     << " "
	    << std::setw(10)      << "[MHz]"     << " "
	    << std::setw(30)      << ""          << " "
	    << std::setw(12)      << "[deg]"     << "\n"
	    << std::setfill('-')  
	    << std::setw(maxsize+1+15+15+31+11+11+13) << "-"
	    << "\n"<< std::setfill(' ') ;

  for(i=0;i<goodlist.size();i++){
    std::cout << std::setw(maxsize) << goodlist[i].name    << " "
	      << std::setprecision(6)
	      << std::setw(14)      << goodlist[i].ra      << " "
	      << std::setw(14)      << goodlist[i].dec     << " "
	      << std::setprecision(4)
	      << std::setw(10)      << goodlist[i].beam    << " "
	      << std::setw(10)      << goodlist[i].obsfreq << " "
	      << std::setw(30)      << goodlist[i].date    << " "
	      << std::setw(12)      << goodlist[i].angsep  << "\n";
  }

}
