#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>
#include <string>
#include <glob.h>
#include "pvo-block.hh"
#include "pvo-header.hh"
#include "pvo-FITS.hh"
#include "pvo-utils.hh"

using namespace PVO;
using namespace PVO::Header;
using namespace PVO::Block;
using namespace PVO::FITS;
using std::string;


int main(int argc, char * argv[])
{
  /** 
   * PVOSummary is a function to print out basic information for
   * PSRFITS files.
   *
   * This program is a command-line program that looks at PSRFITS
   * files (i.e. files that have extensions .fits or .rf) and returns
   * a summary of their positions, beam sizes and observation times.
   *
   * The format of usage is: <br> 
   * <tt>> pvosummary </tt><br>
   * to analyse all PSRFITS files in the current directory, or <br>
   * <tt>> pvosummary FILE</tt><br>
   * to analyse the file FILE. Entering<br>
   * <tt>> pvosummary -h</tt><br>
   * will give a help message, using PVO::pvosummary_usage_text.
   *
   * The output is a list of valid files, their central RA & DEC,
   * their beam size, the central observing frequency, and the
   * DATE-OBS keyword (time/date of observation),
   * 
   */


  bool verboseflag = false;

  std::vector<std::string> namelist;
  unsigned int i;
  switch(argc){
  case 1:
    glob_t globbuf1,globbuf2;
    globbuf1.gl_offs = 0;
    globbuf2.gl_offs = 0;
    glob("*.fits", GLOB_DOOFFS, NULL, &globbuf1);
    glob("*.rf", GLOB_DOOFFS, NULL, &globbuf2);
    namelist.resize(globbuf1.gl_pathc+globbuf2.gl_pathc);
    for(i=0;i<globbuf1.gl_pathc+globbuf1.gl_offs;i++) 
      namelist[i] = globbuf1.gl_pathv[i];
    for(i=0;i<globbuf2.gl_pathc+globbuf2.gl_offs;i++) 
      namelist[globbuf1.gl_pathc+i] = globbuf2.gl_pathv[i];
    break;
  case 2:
    if(strcmp(argv[1],"-h")==0){
      std::cout << pvosummary_usage_text;
      return 0;
    }
    namelist.resize(1);
    namelist[0] = argv[1];
    break;
  default:
    std::cerr << pvosummary_usage_text;
    return 0;
    break;
  }

  int maxsize = 8; // size of "FILENAME"
  for(i=0;i<namelist.size();i++) 
    maxsize = std::max(maxsize,int(namelist[i].size()));

  std::cout << std::setfill('-')  
	    << std::setw(maxsize+1+15+15+31+11+11) << "-"
	    << "\n" << std::setfill(' ') 
	    << std::setw(maxsize) << "FILENAME"  << " "
	    << std::setw(14)      << "RA"        << " "
	    << std::setw(14)      << "DEC"       << " "
	    << std::setw(10)      << "BEAM FWHM" << " "
	    << std::setw(10)      << "OBSFREQ"   << " "
	    << std::setw(30)      << "DATE-OBS"  << "\n"
	    << std::setw(maxsize) << ""          << " "
	    << std::setw(14)      << ""          << " "
	    << std::setw(14)      << ""          << " "
	    << std::setw(10)      << "[deg]"     << " "
	    << std::setw(10)      << "[MHz]"     << " "
	    << std::setw(30)      << ""          << "\n"
	    << std::setfill('-')  
	    << std::setw(maxsize+1+15+15+31+11+11) << "-"
	    << "\n"<< std::setfill(' ') ;

  for(i=0;i<namelist.size();i++){

    PSRFits fitsfile;
    fitsfile.setVerbosity(verboseflag);
    int status = fitsfile.openFile(namelist[i]);
  
    if(status==0){
      fitsfile.gotoPHU();
    
      std::string ra,dec,date,obsfreqS;
      std::string missing = "---";
      if(fitsfile.getKeyword<std::string>("RA",ra)!=0) ra = missing;
      if(fitsfile.getKeyword<std::string>("DEC",dec)!=0) dec = missing;
      if(fitsfile.getKeyword<std::string>("DATE-OBS",date)!=0) date = missing;
      if(fitsfile.getKeyword<std::string>("OBSFREQ",obsfreqS)!=0) obsfreqS = missing;
      float beamradius,bmaj,bmin,obsfreq;
      fitsfile.getKeyword<float>("BMAJ",bmaj);
      fitsfile.getKeyword<float>("BMIN",bmin);
      beamradius = 0.5*(bmaj+bmin); // average FWHM, in deg

      std::cout << std::setw(maxsize) << namelist[i]  << " "
		<< std::setw(14)      << ra           << " "
		<< std::setw(14)      << dec          << " "
		<< std::setw(10)      << beamradius   << " "
		<< std::setw(10)      << obsfreqS     << " "
		<< std::setw(30)      << date         << "\n";
    }
  }

}
